﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Mondol.TaskHosting.Examples.LinkCapture
{
    /// <summary>
    /// 任务 - 采集指定网页内的所有链接及名字
    /// </summary>
    /// <remarks>
    /// 继承自Task的类即为执行入口
    /// 一个程序集内可包含多个继承自Task的类，使用哪个入口由用户在UI中选择确定
    /// </remarks>
    [Task("网页链接采集器", IsMutilThread = true, ConfigType = typeof(Config))]
    class CaptureTask : Mondol.TaskHosting.Task
    {
        readonly Config _cfg;
        readonly IImmediateLogger _immediateLogger;

        string[] _urls;
        int _urlIndex = 0;

        public CaptureTask(Config cfg, IImmediateLogger immediateLogger)
        {
            this._cfg = cfg;
            this._immediateLogger = immediateLogger;
        }

        /// <summary>
        /// 在任务启动前可通过此方法检测配置合法性或做初始化工作
        /// </summary>
        /// <returns>true: 允许继续运行任务 | false: 阻止继续运行任务</returns>
        public override bool OnBeforeRun()
        {
            if (string.IsNullOrWhiteSpace(_cfg.Urls))
            {
                _immediateLogger.Warn("请至少包含一个URL");
                return false;
            }

            _urls = _cfg.Urls.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
            _urlIndex = 0;

            return true;
        }

        private string GetUrl()
        {
            lock (this)
            {
                if (_urlIndex >= _urls.Length)
                    return null;

                return _urls[_urlIndex++];
            }
        }

        public override void OnRun()
        {
            string url;
            while (!CancellationToken.IsCancellationRequested &&
                  (url = GetUrl()) != null)
            {
                var wc = new System.Net.WebClient();
                wc.Encoding = Encoding.UTF8;
                var html = wc.DownloadString(url);
                CancellationToken.ThrowIfCancellationRequested(); //检测用户取消

                var rexLink = new Regex(@"<a\s+[^>]*href=""([^""]+)[^>]*>([^<]+)</a>", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                var linkCount = 0;
                var mc = rexLink.Matches(html);
                foreach (Match m in mc)
                {
                    var linkUrl = m.Groups[1].Value;
                    var linkName = m.Groups[2].Value.Trim();
                    if (!linkUrl.StartsWith("http://", StringComparison.OrdinalIgnoreCase) &&
                        !linkUrl.StartsWith("https://", StringComparison.OrdinalIgnoreCase))
                        continue;

                    linkCount++;

                    //immediateLogger的Info,Warn,Debug等方法对应UI中的日志级别
                    //用户可以按日志级别过滤决定显示的日志条目
                    _immediateLogger.Debug($"已采集到链接 {linkName} = {linkUrl}");
                }

                _immediateLogger.Info($"Url={url}的网址已采集到了 {linkCount} 个链接");
            }
        }
    }
}
